@@ -1,5 +1,3 @@
-from __future__ import print_function, division, absolute_import
-
 from collections import defaultdict
 import logging
 from timeit import default_timer
@@ -74,12 +72,11 @@ class Progress(SchedulerPlugin):
         self.status = None
         self.extra = {}
 
-    @gen.coroutine
-    def setup(self):
+    async def setup(self):
         keys = self.keys
 
         while not keys.issubset(self.scheduler.tasks):
-            yield gen.sleep(0.05)
+            await gen.sleep(0.05)
 
         tasks = [self.scheduler.tasks[k] for k in keys]
 
@@ -163,12 +160,11 @@ class MultiProgress(Progress):
             self, keys, scheduler, minimum=minimum, dt=dt, complete=complete
         )
 
-    @gen.coroutine
-    def setup(self):
+    async def setup(self):
         keys = self.keys
 
         while not keys.issubset(self.scheduler.tasks):
-            yield gen.sleep(0.05)
+            await gen.sleep(0.05)
 
         tasks = [self.scheduler.tasks[k] for k in keys]
 
